@@ -23,10 +23,8 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-import codecs
-
 from gnomemusic.gstplayer import Playback
-from gnomemusic.player import PlayerField, PlayerPlaylist, RepeatMode
+from gnomemusic.player import PlayerPlaylist, RepeatMode
 from gnomemusic.grilo import grilo
 from gnomemusic.playlists import Playlists
 from gnomemusic.utils import View
@@ -224,12 +222,13 @@ class MediaPlayer2Service(Server):
         self.player = app.get_active_window()._player
         self.player.connect(
             'song-changed', self._on_current_song_changed)
-        self.player.connect('playback-status-changed', self._on_playback_status_changed)
+        self.player.connect('notify::state', self._on_player_state_changed)
         self.player.connect('notify::repeat-mode', self._on_repeat_mode_changed)
         self.player.connect('volume-changed', self._on_volume_changed)
         self.player.connect('prev-next-invalidated', self._on_prev_next_invalidated)
-        self.player.connect('seeked', self._on_seeked)
-        self.player.connect('playlist-changed', self._on_playlist_changed)
+        self.player.connect('seek-finished', self._on_seek_finished)
+        self.player.connect(
+            'playlist-changed', self._on_player_playlist_changed)
         self.player_toolbar = app.get_active_window()._player_toolbar
         self.player_toolbar.connect(
             'thumbnail-updated', self._on_thumbnail_updated)
@@ -238,11 +237,13 @@ class MediaPlayer2Service(Server):
         playlists.connect('playlist-deleted', self._on_playlists_count_changed)
         grilo.connect('ready', self._on_grilo_ready)
         self.playlists = []
-        self.first_song_handler = 0
+        self._player_previous_type = None
+        self._path_list = []
+        self._metadata_list = []
 
     @log
     def _get_playback_status(self):
-        state = self.player.get_playback_status()
+        state = self.player.props.state
         if state == Playback.PLAYING:
             return 'Playing'
         elif state == Playback.PAUSED:
@@ -266,8 +267,9 @@ class MediaPlayer2Service(Server):
         if not media:
             return {}
 
+        song_dbus_path = self._get_song_dbus_path(media)
         metadata = {
-            'mpris:trackid': GLib.Variant('o', self._get_media_id(media)),
+            'mpris:trackid': GLib.Variant('o', song_dbus_path),
             'xesam:url': GLib.Variant('s', media.get_url())
         }
 
@@ -338,41 +340,63 @@ class MediaPlayer2Service(Server):
         return metadata
 
     @log
-    def _get_media_id(self, media):
-        if media:
-            trackid = "/org/gnome/GnomeMusic/Tracklist/{}".format(
-                codecs.encode(
-                    bytes(media.get_id(), 'ascii'), 'hex').decode('ascii'))
+    def _get_song_dbus_path(self, media):
+        """Convert a Grilo media to a D-Bus path
+
+        The hex encoding is used to remove any possible invalid character.
+
+        :param Grl.Media media: The media object
+        :return: a D-Bus id to uniquely identify the song
+        :rtype: str
+        """
+        if not media:
+            return "/org/mpris/MediaPlayer2/TrackList/NoTrack"
+
+        id_hex = media.get_id().encode('ascii').hex()
+        path = "/org/gnome/GnomeMusic/TrackList/{}".format(id_hex)
+        return path
+
+    @log
+    def _update_songs_list(self):
+        previous_path_list = self._path_list
+        self._path_list = []
+        self._metadata_list = []
+        for song in self.player.get_mpris_playlist():
+            path = self._get_song_dbus_path(song)
+            metadata = self._get_metadata(song)
+            self._path_list.append(path)
+            self._metadata_list.append(metadata)
+
+        # current song has changed
+        if (not previous_path_list
+                or previous_path_list[0] != self._path_list[0]
+                or previous_path_list[-1] != self._path_list[-1]):
+            current_song_path = self._get_song_dbus_path(
+                self.player.props.current_song)
+            self.TrackListReplaced(self._path_list, current_song_path)
+            self.PropertiesChanged(
+                MediaPlayer2Service.MEDIA_PLAYER2_TRACKLIST_IFACE,
+                {'Tracks': GLib.Variant('ao', self._path_list), }, [])
+
+    @log
+    def _get_playlist_dbus_path(self, playlist):
+        """Convert a playlist to a D-Bus path
+
+        :param Grl.media playlist: The playlist object
+        :return: a D-Bus id to uniquely identify the playlist
+        :rtype: str
+        """
+        if playlist:
+            id_ = playlist.get_id()
         else:
-            trackid = "/org/mpris/MediaPlayer2/TrackList/NoTrack"
+            id_ = 'Invalid'
 
-        return trackid
+        return '/org/mpris/MediaPlayer2/Playlist/{}'.format(id_)
 
     @log
-    def _get_media_from_id(self, track_id):
-        for track in self.player.get_songs():
-            media = track[PlayerField.SONG]
-            if track_id == self._get_media_id(media):
-                return media
-        return None
-
-    @log
-    def _get_track_list(self):
-        if self.player.props.playing:
-            return [self._get_media_id(song[PlayerField.SONG])
-                    for song in self.player.get_songs()]
-        else:
-            return []
-
-    @log
-    def _get_playlist_path(self, playlist):
-        return '/org/mpris/MediaPlayer2/Playlist/%s' % \
-            (playlist.get_id() if playlist else 'Invalid')
-
-    @log
-    def _get_playlist_from_path(self, playlist_path):
+    def _get_playlist_from_dbus_path(self, playlist_path):
         for playlist in self.playlists:
-            if playlist_path == self._get_playlist_path(playlist):
+            if playlist_path == self._get_playlist_dbus_path(playlist):
                 return playlist
         return None
 
@@ -397,18 +421,27 @@ class MediaPlayer2Service(Server):
 
     @log
     def _get_active_playlist(self):
-        playlist = None
-        playlist_name = ''
-        if self.player.get_playlist_type() == PlayerPlaylist.Type.PLAYLIST:
-            playlist = self._get_playlist_from_id(
-                self.player.get_playlist_id())
-            playlist_name = utils.get_media_title(playlist)
+        """Get Active Maybe_Playlist
 
-        path = self._get_playlist_path(playlist)
-        return (playlist is not None, (path, playlist_name, ''))
+        Maybe_Playlist is a structure describing a playlist, or nothing
+        according to MPRIS specifications.
+        If a playlist is active, return True and its description
+        (path, name and icon).
+        If no playlist is active, return False and an undefined structure.
+        :returns: playlist existence and its structure
+        :rtype: tuple
+        """
+        if self.player.get_playlist_type() != PlayerPlaylist.Type.PLAYLIST:
+            return (False, ("/", "", ""))
+
+        playlist = self._get_playlist_from_id(self.player.get_playlist_id())
+        playlist_name = utils.get_media_title(playlist)
+        path = self._get_playlist_dbus_path(playlist)
+        return (True, (path, playlist_name, ""))
 
     @log
     def _on_current_song_changed(self, player, position):
+        self._update_songs_list()
         if self.player.props.repeat_mode == RepeatMode.SONG:
             self.Seeked(0)
 
@@ -429,7 +462,7 @@ class MediaPlayer2Service(Server):
                                [])
 
     @log
-    def _on_playback_status_changed(self, data=None):
+    def _on_player_state_changed(self, klass, args):
         self.PropertiesChanged(MediaPlayer2Service.MEDIA_PLAYER2_PLAYER_IFACE,
                                {
                                    'PlaybackStatus': GLib.Variant('s', self._get_playback_status()),
@@ -438,6 +471,7 @@ class MediaPlayer2Service(Server):
 
     @log
     def _on_repeat_mode_changed(self, player, param):
+        self._update_songs_list()
         self.PropertiesChanged(MediaPlayer2Service.MEDIA_PLAYER2_PLAYER_IFACE,
                                {
                                    'LoopStatus': GLib.Variant('s', self._get_loop_status()),
@@ -463,38 +497,21 @@ class MediaPlayer2Service(Server):
                                [])
 
     @log
-    def _play_first_song(self, model, path, iter_, data=None):
-        if self.first_song_handler:
-            model.disconnect(self.first_song_handler)
-            self.first_song_handler = 0
-        self.player.set_playlist(PlayerPlaylist.Type.SONGS, None, model, iter_)
-        self.player.play()
+    def _on_seek_finished(self, player, position_second):
+        self.Seeked(int(position_second * 1e6))
 
     @log
-    def _on_seeked(self, player, position, data=None):
-        self.Seeked(position)
+    def _on_player_playlist_changed(self, klass):
+        self._update_songs_list()
 
-    @log
-    def _on_playlist_changed(self, player, data=None):
-        self._on_playlist_modified()
+        if (self.player.get_playlist_type() == PlayerPlaylist.Type.PLAYLIST
+                or self._player_previous_type == PlayerPlaylist.Type.PLAYLIST):
+            variant = GLib.Variant('(b(oss))', self._get_active_playlist())
+            self.PropertiesChanged(
+                MediaPlayer2Service.MEDIA_PLAYER2_PLAYLISTS_IFACE,
+                {'ActivePlaylist':  variant, }, [])
 
-        self.PropertiesChanged(MediaPlayer2Service.MEDIA_PLAYER2_PLAYLISTS_IFACE,
-                               {
-                                'ActivePlaylist': GLib.Variant('(b(oss))', self._get_active_playlist()),
-                               },
-                               [])
-
-    @log
-    def _on_playlist_modified(self, path=None, _iter=None, data=None):
-        if self.player.props.current_song:
-            track_list = self._get_track_list()
-            self.TrackListReplaced(
-                track_list, self._get_media_id(self.player.props.current_song))
-            self.PropertiesChanged(MediaPlayer2Service.MEDIA_PLAYER2_TRACKLIST_IFACE,
-                                   {
-                                       'Tracks': GLib.Variant('ao', track_list),
-                                   },
-                                   [])
+        self._player_previous_type = klass.get_playlist_type()
 
     @log
     def _reload_playlists(self):
@@ -538,53 +555,72 @@ class MediaPlayer2Service(Server):
         self.player.stop()
 
     def Play(self):
-        if self.player.get_songs():
-            self.player.play()
-        elif self.first_song_handler == 0:
-            window = self.app.get_active_window()
-            window._stack.set_visible_child(window.views[View.SONG])
-            model = window.views[View.SONG].model
-            if model.iter_n_children(None):
-                _iter = model.get_iter_first()
-                self._play_first_song(model, model.get_path(_iter), _iter)
-            else:
-                self.first_song_handler = model.connect('row-inserted', self._play_first_song)
+        """Start or resume playback.
 
-    def Seek(self, offset):
-        self.player.set_position(offset, True, True)
+        If there is no track to play, this has no effect.
+        """
+        self.player.play()
 
-    def SetPosition(self, track_id, position):
+    def Seek(self, offset_msecond):
+        """Seek forward in the current track.
+
+        Seek is relative to the current player position.
+        If the value passed in would mean seeking beyond the end of the track,
+        acts like a call to Next.
+        :param int offset_msecond: number of microseconds
+        """
+        current_position_second = self.player.get_position()
+        new_position_second = current_position_second + offset_msecond / 1e6
+
+        duration_second = self.player.props.duration
+        if new_position_second <= duration_second:
+            self.player.set_position(new_position_second)
+        else:
+            self.player.next()
+
+    def SetPosition(self, track_id, position_msecond):
+        """Set the current track position in microseconds.
+
+        :param str track_id: The currently playing track's identifier
+        :param int position_msecond: new position in microseconds
+        """
         if track_id != self._get_metadata().get('mpris:trackid').get_string():
             return
-        self.player.set_position(position)
+        self.player.set_position(position_msecond / 1e6)
 
     def OpenUri(self, uri):
         pass
 
-    def Seeked(self, position):
-        self.con.emit_signal(None,
-                             '/org/mpris/MediaPlayer2',
-                             MediaPlayer2Service.MEDIA_PLAYER2_PLAYER_IFACE,
-                             'Seeked',
-                             GLib.Variant.new_tuple(GLib.Variant('x', position)))
+    def Seeked(self, position_msecond):
+        """Indicate that the track position has changed.
 
-    def GetTracksMetadata(self, track_ids):
+        :param int position_msecond: new position in microseconds.
+        """
+        variant = GLib.Variant.new_tuple(GLib.Variant('x', position_msecond))
+        self.con.emit_signal(
+            None, '/org/mpris/MediaPlayer2',
+            MediaPlayer2Service.MEDIA_PLAYER2_PLAYER_IFACE, 'Seeked', variant)
+
+    def GetTracksMetadata(self, track_paths):
         metadata = []
-        for track_id in track_ids:
-            metadata.append(self._get_metadata(self._get_media_from_id(track_id)))
+        for path in track_paths:
+            index = self._path_list.index(path)
+            metadata.append(self._metadata_list[index])
         return metadata
 
     def AddTrack(self, uri, after_track, set_as_current):
         pass
 
-    def RemoveTrack(self, track_id):
+    def RemoveTrack(self, path):
         pass
 
-    def GoTo(self, track_id):
-        for index, song in enumerate(self.player.get_songs()):
-            if track_id == self._get_media_id(song[PlayerField.SONG]):
-                self.player_play(index)
-                return
+    def GoTo(self, path):
+        current_song_path = self._get_song_dbus_path(
+            self.player.props.current_song)
+        current_song_index = self._path_list.index(current_song_path)
+        goto_index = self._path_list.index(path)
+        self.player.play(goto_index - current_song_index)
+        return
 
     def TrackListReplaced(self, tracks, current_song):
         self.con.emit_signal(None,
@@ -602,29 +638,28 @@ class MediaPlayer2Service(Server):
                              GLib.Variant.new_tuple(GLib.Variant('a{sv}', metadata),
                                                     GLib.Variant('o', after_track)))
 
-    def TrackRemoved(self, track_id):
-        self.con.emit_signal(None,
-                             '/org/mpris/MediaPlayer2',
-                             MediaPlayer2Service.MEDIA_PLAYER2_TRACKLIST_IFACE,
-                             'TrackRemoved',
-                             GLib.Variant.new_tuple(GLib.Variant('o', track_id)))
+    def TrackRemoved(self, path):
+        self.con.emit_signal(
+            None, '/org/mpris/MediaPlayer2',
+            MediaPlayer2Service.MEDIA_PLAYER2_TRACKLIST_IFACE, 'TrackRemoved',
+            GLib.Variant.new_tuple(GLib.Variant('o', path)))
 
-    def TrackMetadataChanged(self, track_id, metadata):
-        self.con.emit_signal(None,
-                             '/org/mpris/MediaPlayer2',
-                             MediaPlayer2Service.MEDIA_PLAYER2_TRACKLIST_IFACE,
-                             'TrackMetadataChanged',
-                             GLib.Variant.new_tuple(GLib.Variant('o', track_id),
-                                                    GLib.Variant('a{sv}', metadata)))
+    def TrackMetadataChanged(self, path, metadata):
+        self.con.emit_signal(
+            None, '/org/mpris/MediaPlayer2',
+            MediaPlayer2Service.MEDIA_PLAYER2_TRACKLIST_IFACE,
+            'TrackMetadataChanged',
+            GLib.Variant.new_tuple(
+                GLib.Variant('o', path), GLib.Variant('a{sv}', metadata)))
 
     def ActivatePlaylist(self, playlist_path):
-        playlist_id = self._get_playlist_from_path(playlist_path).get_id()
+        playlist_id = self._get_playlist_from_dbus_path(playlist_path).get_id()
         self.app._window.views[View.PLAYLIST].activate_playlist(playlist_id)
 
     def GetPlaylists(self, index, max_count, order, reverse):
         if order != 'Alphabetical':
             return []
-        playlists = [(self._get_playlist_path(playlist),
+        playlists = [(self._get_playlist_dbus_path(playlist),
                       utils.get_media_title(playlist) or '', '')
                      for playlist in self.playlists]
         return playlists[index:index + max_count] if not reverse \
@@ -661,6 +696,7 @@ class MediaPlayer2Service(Server):
                 ]),
             }
         elif interface_name == MediaPlayer2Service.MEDIA_PLAYER2_PLAYER_IFACE:
+            position_msecond = int(self.player.get_position() * 1e6)
             return {
                 'PlaybackStatus': GLib.Variant('s', self._get_playback_status()),
                 'LoopStatus': GLib.Variant('s', self._get_loop_status()),
@@ -668,7 +704,7 @@ class MediaPlayer2Service(Server):
                 'Shuffle': GLib.Variant('b', self.player.props.repeat_mode == RepeatMode.SHUFFLE),
                 'Metadata': GLib.Variant('a{sv}', self._get_metadata()),
                 'Volume': GLib.Variant('d', self.player.get_volume()),
-                'Position': GLib.Variant('x', self.player.get_position()),
+                'Position': GLib.Variant('x', position_msecond),
                 'MinimumRate': GLib.Variant('d', 1.0),
                 'MaximumRate': GLib.Variant('d', 1.0),
                 'CanGoNext': GLib.Variant('b', self.player.props.has_next),
@@ -680,7 +716,7 @@ class MediaPlayer2Service(Server):
             }
         elif interface_name == MediaPlayer2Service.MEDIA_PLAYER2_TRACKLIST_IFACE:
             return {
-                'Tracks': GLib.Variant('ao', self._get_track_list()),
+                'Tracks': GLib.Variant('ao', self._path_list),
                 'CanEditTracks': GLib.Variant('b', False)
             }
         elif interface_name == MediaPlayer2Service.MEDIA_PLAYER2_PLAYLISTS_IFACE:
