@@ -24,6 +24,7 @@
 
 from collections import defaultdict
 from enum import IntEnum
+from itertools import chain
 from random import shuffle, randrange
 import logging
 import time
@@ -38,6 +39,7 @@ from gnomemusic import log
 from gnomemusic.gstplayer import GstPlayer, Playback
 from gnomemusic.grilo import grilo
 from gnomemusic.inhibitsuspend import InhibitSuspend
+from gnomemusic.pauseonsuspend import PauseOnSuspend
 from gnomemusic.playlists import Playlists
 from gnomemusic.scrobbler import LastFmScrobbler
 
@@ -86,6 +88,8 @@ class PlayerPlaylist(GObject.GObject):
         'song-validated': (GObject.SignalFlags.RUN_FIRST, None, (int, int)),
     }
 
+    _nb_songs_max = 10
+
     def __repr__(self):
         return '<PlayerPlayList>'
 
@@ -160,14 +164,24 @@ class PlayerPlaylist(GObject.GObject):
         return True
 
     @log
-    def set_song(self, song_index):
+    def set_song(self, song_offset):
         """Change playlist index.
 
-        :param int song_index: requested song index
+        :param int song_offset: position relative to current song
         :return: True if the index has changed. False otherwise.
         :rtype: bool
         """
-        if song_index >= len(self._songs):
+        if self._repeat == RepeatMode.SHUFFLE:
+            shuffle = self._shuffle_indexes.index(self._current_index)
+            self._current_index = self._shuffle_indexes[shuffle + song_offset]
+            return True
+
+        song_index = song_offset + self._current_index
+        if self._repeat == RepeatMode.ALL:
+            song_index = song_index % len(self._songs)
+
+        if(song_index >= len(self._songs)
+           or song_index < 0):
             return False
 
         self._current_index = song_index
@@ -313,8 +327,8 @@ class PlayerPlaylist(GObject.GObject):
                 and self._current_index == 0):
             return len(self._songs) - 1
         if self._repeat == RepeatMode.SHUFFLE:
-            index = self._shuffle_indexes.index(self._current_index)
-            return self._shuffle_indexes[index - 1]
+            shuffle_index = self._shuffle_indexes.index(self._current_index)
+            return self._shuffle_indexes[shuffle_index - 1]
         else:
             return self._current_index - 1
 
@@ -477,16 +491,54 @@ class PlayerPlaylist(GObject.GObject):
         return self._type
 
     @log
-    def get_songs(self):
-        """Get the current playlist.
+    def get_mpris_playlist(self):
+        """Get recent and next songs from the current playlist.
 
-        Each member of the list has two elements: the song, and the validation
-        status.
+        If the playlist is an album, return all songs.
+        Returned songs are sorted according to the repeat mode.
+        This method is used by mpris to expose a TrackList.
 
         :returns: current playlist
-        :rtype: list
+        :rtype: list of Grl.Media
         """
-        return self._songs
+        if not self.props.current_song:
+            return []
+
+        songs = []
+        nb_songs = len(self._songs)
+        current_index = self._current_index
+        if self._repeat == RepeatMode.SHUFFLE:
+            current_index = self._shuffle_indexes.index(self._current_index)
+
+        index_min = current_index - self._nb_songs_max
+        index_max = current_index + self._nb_songs_max + 1
+        if self._type == PlayerPlaylist.Type.ALBUM:
+            index_min = 0
+            index_max = nb_songs
+
+        first_index = max(index_min, 0)
+        last_index = min(index_max, nb_songs)
+
+        if self._repeat == RepeatMode.SHUFFLE:
+            indexes = self._shuffle_indexes[first_index:last_index]
+        else:
+            indexes = range(first_index, last_index)
+
+        if (self._repeat == RepeatMode.ALL
+                and (last_index - first_index) < (2 * self._nb_songs_max + 1)):
+            offset_sup = min(
+                self._nb_songs_max - last_index + current_index + 1,
+                first_index)
+            offset_inf = min(
+                self._nb_songs_max - current_index + first_index,
+                nb_songs - last_index)
+
+            indexes = chain(
+                range(nb_songs - offset_inf, nb_songs), indexes,
+                range(offset_sup))
+
+        songs = [self._songs[index][PlayerField.SONG] for index in indexes]
+        return songs
 
 
 class Player(GObject.GObject):
@@ -497,15 +549,17 @@ class Player(GObject.GObject):
 
     __gsignals__ = {
         'clock-tick': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
-        'playback-status-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'playlist-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'prev-next-invalidated': (GObject.SignalFlags.RUN_FIRST, None, ()),
-        'seeked': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
+        'seek-finished': (GObject.SignalFlags.RUN_FIRST, None, (float,)),
         'song-changed': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
         'song-validated': (GObject.SignalFlags.RUN_FIRST, None, (int, int)),
         'volume-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
+    state = GObject.Property(type=int, default=Playback.STOPPED)
+    duration = GObject.Property(type=float, default=-1.)
+
     def __repr__(self):
         return '<Player>'
 
@@ -519,20 +573,25 @@ class Player(GObject.GObject):
         self._playlist.connect('song-validated', self._on_song_validated)
         self._playlist.bind_property(
             'repeat-mode', self, 'repeat-mode',
-            GObject.BindingFlags.SYNC_CREATE |
-            GObject.BindingFlags.BIDIRECTIONAL)
+            GObject.BindingFlags.SYNC_CREATE
+            | GObject.BindingFlags.BIDIRECTIONAL)
 
         self._new_clock = True
 
         Gst.init(None)
         GstPbutils.pb_utils_init()
 
-        self._player = GstPlayer()
-        self._player.connect('clock-tick', self._on_clock_tick)
-        self._player.connect('eos', self._on_eos)
+        self._gst_player = GstPlayer()
+        self._gst_player.connect('clock-tick', self._on_clock_tick)
+        self._gst_player.connect('eos', self._on_eos)
+        self._gst_player.bind_property(
+            'duration', self, 'duration', GObject.BindingFlags.SYNC_CREATE)
+        self._gst_player.bind_property(
+            'state', self, 'state', GObject.BindingFlags.SYNC_CREATE)
 
         root_window = parent_window.get_toplevel()
         self._inhibit_suspend = InhibitSuspend(root_window, self)
+        self._pause_on_suspend = PauseOnSuspend(self)
 
         self._lastfm = LastFmScrobbler()
 
@@ -564,16 +623,16 @@ class Player(GObject.GObject):
         :returns: True if a song is currently played.
         :rtype: bool
         """
-        return self._player.state == Playback.PLAYING
+        return self.props.state == Playback.PLAYING
 
     @log
     def _load(self, song):
-        self._player.state = Playback.LOADING
+        self._gst_player.props.state = Playback.LOADING
         self._time_stamp = int(time.time())
 
         url_ = song.get_url()
-        if url_ != self._player.url:
-            self._player.url = url_
+        if url_ != self._gst_player.props.url:
+            self._gst_player.props.url = url_
 
         self.emit('song-changed', self._playlist.get_current_index())
 
@@ -589,32 +648,35 @@ class Player(GObject.GObject):
             self.stop()
 
     @log
-    def play(self, song_index=None):
-        """Play"""
-        if not self._playlist:
+    def play(self, song_offset=None):
+        """Play a song.
+
+        If song_offset is None, load and play current song. Otherwise, load a
+        new song and play it.
+
+        :param int song_offset: position relative to current song
+        """
+        if self.props.current_song is None:
             return
 
-        if (song_index
-                and not self._playlist.set_song(song_index)):
+        if (song_offset is not None
+                and not self._playlist.set_song(song_offset)):
             return False
 
-        if self._player.state != Playback.PAUSED:
+        if self.props.state != Playback.PAUSED:
             self._load(self._playlist.props.current_song)
 
-        self._player.state = Playback.PLAYING
-        self.emit('playback-status-changed')
+        self._gst_player.props.state = Playback.PLAYING
 
     @log
     def pause(self):
         """Pause"""
-        self._player.state = Playback.PAUSED
-        self.emit('playback-status-changed')
+        self._gst_player.props.state = Playback.PAUSED
 
     @log
     def stop(self):
         """Stop"""
-        self._player.state = Playback.STOPPED
-        self.emit('playback-status-changed')
+        self._gst_player.props.state = Playback.STOPPED
 
     @log
     def next(self):
@@ -622,7 +684,6 @@ class Player(GObject.GObject):
 
         Play the next song of the playlist, if any.
         """
-
         if self._playlist.next():
             self.play()
 
@@ -632,10 +693,9 @@ class Player(GObject.GObject):
 
         Play the previous song of the playlist, if any.
         """
-        position = self._player.position
+        position = self._gst_player.props.position
         if position >= 5:
-            self._player.seek(0)
-            self._player.state = Playback.PLAYING
+            self.set_position(0.0)
             return
 
         if self._playlist.previous():
@@ -644,7 +704,7 @@ class Player(GObject.GObject):
     @log
     def play_pause(self):
         """Toggle play/pause state"""
-        if self._player.state == Playback.PLAYING:
+        if self.props.state == Playback.PLAYING:
             self.pause()
         else:
             self.play()
@@ -661,7 +721,7 @@ class Player(GObject.GObject):
         playlist_changed = self._playlist.set_playlist(
             playlist_type, playlist_id, model, iter_)
 
-        if self._player.state == Playback.PLAYING:
+        if self.props.state == Playback.PLAYING:
             self.emit('prev-next-invalidated')
 
         if playlist_changed:
@@ -730,7 +790,7 @@ class Player(GObject.GObject):
     @log
     def _on_clock_tick(self, klass, tick):
         logger.debug("Clock tick {}, player at {} seconds".format(
-            tick, self._player.position))
+            tick, self._gst_player.props.position))
 
         current_song = self._playlist.props.current_song
 
@@ -738,15 +798,14 @@ class Player(GObject.GObject):
             self._new_clock = True
             self._lastfm.now_playing(current_song)
 
-        duration = self._player.duration
-        if duration is None:
+        if self.props.duration == -1.:
             return
 
-        position = self._player.position
+        position = self._gst_player.props.position
         if position > 0:
-            percentage = tick / duration
+            percentage = tick / self.props.duration
             if (not self._lastfm.scrobbled
-                    and duration > 30
+                    and self.props.duration > 30.
                     and (percentage > 0.5 or tick > 4 * 60)):
                 self._lastfm.scrobble(current_song, self._time_stamp)
 
@@ -779,8 +838,6 @@ class Player(GObject.GObject):
         :returns: the song being played. None if there is no playlist.
         :rtype: Grl.Media
         """
-        if not self._playlist:
-            return None
         return self._playlist.props.current_song
 
     @log
@@ -801,59 +858,51 @@ class Player(GObject.GObject):
         """
         return self._playlist.props.playlist_id
 
-    # MPRIS
-    @log
-    def get_gst_player(self):
-        """GstPlayer getter"""
-        return self._player
-
-    @log
-    def get_playback_status(self):
-        # FIXME: Just a proxy right now.
-        return self._player.state
-
-    @GObject.Property
-    def url(self):
-        """GstPlayer url loaded
-
-        :return: url
-        :rtype: string
-        """
-        # FIXME: Just a proxy right now.
-        return self._player.url
-
     @log
     def get_position(self):
-        return self._player.position
+        """Get player position.
+
+        Player position in seconds.
+        :returns: position
+        :rtype: float
+        """
+        return self._gst_player.props.position
 
     # TODO: used by MPRIS
     @log
-    def set_position(self, offset, start_if_ne=False, next_on_overflow=False):
-        if offset < 0:
-            if start_if_ne:
-                offset = 0
-            else:
-                return
+    def set_position(self, position_second):
+        """Change GstPlayer position.
 
-        duration = self._player.duration
-        if duration is None:
-            return
+        If the position if negative, set it to zero.
+        If the position if greater than song duration, do nothing
+        :param float position_second: requested position in second
+        """
+        if position_second < 0.0:
+            position_second = 0.0
 
-        if duration >= offset * 1000:
-            self._player.seek(offset * 1000)
-            self.emit('seeked', offset)
-        elif next_on_overflow:
-            self.next()
+        duration_second = self._gst_player.props.duration
+        if position_second <= duration_second:
+            self._gst_player.seek(position_second)
+            self.emit('seek-finished', position_second)
 
     @log
     def get_volume(self):
-        return self._player.volume
+        return self._gst_player.props.volume
 
     @log
     def set_volume(self, rate):
-        self._player.volume = rate
+        self._gst_player.props.volume = rate
         self.emit('volume-changed')
 
     @log
-    def get_songs(self):
-        return self._playlist.get_songs()
+    def get_mpris_playlist(self):
+        """Get recent and next songs from the current playlist.
+
+        If the playlist is an album, return all songs.
+        Returned songs are sorted according to the repeat mode.
+        This method is used by mpris to expose a TrackList.
+
+        :returns: current playlist
+        :rtype: list of Grl.Media
+        """
+        return self._playlist.get_mpris_playlist()
