@@ -87,8 +87,8 @@ class AlbumWidget(Gtk.EventBox):
 
         self.bind_property(
             'selection-mode', self._parent_view, 'selection-mode',
-            GObject.BindingFlags.BIDIRECTIONAL |
-            GObject.BindingFlags.SYNC_CREATE)
+            GObject.BindingFlags.BIDIRECTIONAL
+            | GObject.BindingFlags.SYNC_CREATE)
 
         self.bind_property(
             'selected-items-count', self._parent_view, 'selected-items-count')
@@ -154,6 +154,8 @@ class AlbumWidget(Gtk.EventBox):
 
         if composer:
             self._composer_info_label.props.label = composer
+            self._composer_info_label.props.max_width_chars = 10
+            self._composer_info_label.props.tooltip_text = composer
             show = True
 
         self._composer_label.props.visible = show
