@@ -37,16 +37,9 @@ class SmoothScale(Gtk.Scale):
 
     The progressbar Gtk.Scale, extended with smoothing capabilities, so
     the indicator updates every pixel available.
-    This class interacts directly with the GstPlayer class.
     """
     __gtype_name__ = 'SmoothScale'
 
-    __gsignals__ = {
-        'seek-finished': (
-            GObject.SignalFlags.RUN_FIRST, None, (float,)
-        ),
-    }
-
     def __repr__(self):
         return '<SmoothScale>'
 
@@ -62,28 +55,35 @@ class SmoothScale(Gtk.Scale):
 
         self._timeout = None
 
-        self.connect('button-press-event', self._on_smooth_scale_event)
-        self.connect(
-            'button-release-event', self._on_smooth_scale_button_released)
+        self._controller = Gtk.GestureMultiPress().new(self)
+        self._controller.props.propagation_phase = Gtk.PropagationPhase.CAPTURE
+        self._controller.connect("pressed", self._on_button_pressed)
+        self._controller.connect("released", self._on_button_released)
+
         self.connect('change-value', self._on_smooth_scale_seek)
 
     # FIXME: This is a workaround for not being able to pass the player
     # object via init when using Gtk.Builder.
     @GObject.Property
     def player(self):
-        """The GstPlayer object used
+        """The Player object used
 
         :return: player object
-        :rtype: GstPlayer
+        :rtype: Player
         """
         return self._player
 
     @player.setter
     def player(self, player):
-        """Set the GstPlayer object used
+        """Set the Player object used
 
-        :param GstPlayer player: The GstPlayer to use
+        :param Player player: The Player to use
         """
+        if (player is None
+                or (self._player is not None
+                    and self._player != player)):
+            return
+
         self._player = player
 
         self._player.connect('notify::state', self._on_state_change)
@@ -91,18 +91,19 @@ class SmoothScale(Gtk.Scale):
 
     @log
     def _on_state_change(self, klass, arguments):
-        state = self._player.state
+        state = self._player.props.state
 
         self._previous_state = state
 
-        if state == Playback.STOPPED:
+        if (state == Playback.STOPPED
+                or state == Playback.LOADING):
             self.set_value(0)
-            self.set_sensitive(False)
+            self.props.sensitive = False
         else:
-            self.set_sensitive(True)
+            self.props.sensitive = True
 
         if state == Playback.PLAYING:
-            self._update_timeout()
+            self._update_position_callback()
         else:
             self._remove_timeout()
 
@@ -110,9 +111,9 @@ class SmoothScale(Gtk.Scale):
 
     @log
     def _on_duration_changed(self, klass, arguments):
-        duration = self._player.duration
+        duration = self._player.props.duration
 
-        if duration is not None:
+        if duration != -1.:
             self.set_range(0.0, duration * 60)
             self.set_increments(300, 600)
 
@@ -120,12 +121,11 @@ class SmoothScale(Gtk.Scale):
     def _on_smooth_scale_seek_finish(self, value):
         """Prevent stutters when seeking with infinitesimal amounts"""
         self._seek_timeout = None
-        round_digits = self.get_property('round-digits')
+        round_digits = self.props.round_digits
         if self._old_smooth_scale_value != round(value, round_digits):
             self._on_smooth_scale_change_value(self)
             self._old_smooth_scale_value = round(value, round_digits)
 
-        self.emit('seek-finished', value)
         return False
 
     @log
@@ -150,19 +150,21 @@ class SmoothScale(Gtk.Scale):
         return True
 
     @log
-    def _on_smooth_scale_button_released(self, scale, data):
+    def _on_button_released(self, gesture, n_press, x, y):
         if self._seek_timeout:
             GLib.source_remove(self._seek_timeout)
             self._on_smooth_scale_seek_finish(
                 self.get_value())
 
         self._update_position_callback()
+
         return False
 
     @log
-    def _on_smooth_scale_event(self, scale, data):
+    def _on_button_pressed(self, gesture, n_press, x, y):
         self._remove_timeout()
         self._old_smooth_scale_value = self.get_value()
+
         return False
 
     @log
@@ -175,9 +177,9 @@ class SmoothScale(Gtk.Scale):
         """
         # Do not run until SmoothScale has been realized and GStreamer
         # provides a duration.
-        duration = self._player.duration
+        duration = self._player.props.duration
         if (self.get_realized() is False
-                or duration is None):
+                or duration == -1.):
             return
 
         # Update self._timeout.
@@ -202,13 +204,13 @@ class SmoothScale(Gtk.Scale):
     @log
     def _on_smooth_scale_change_value(self, scroll):
         seconds = scroll.get_value() / 60
-        self._player.seek(seconds)
+        self._player.set_position(seconds)
 
         return True
 
     @log
     def _update_position_callback(self):
-        position = self._player.position
+        position = self._player.get_position()
         if position > 0:
             self.set_value(position * 60)
         self._update_timeout()
