@@ -217,6 +217,12 @@ class Window(Gtk.ApplicationWindow):
         self._key_press_event_id = self.connect(
             'key_press_event', self._on_key_press)
 
+        self._btn_ctrl = Gtk.GestureMultiPress().new(self)
+        self._btn_ctrl.props.propagation_phase = Gtk.PropagationPhase.CAPTURE
+        # Mouse button 8 is the back button.
+        self._btn_ctrl.props.button = 8
+        self._btn_ctrl.connect("pressed", self._on_back_button_pressed)
+
         # FIXME: In case Grilo is already initialized before the views
         # get created, they never receive a 'ready' signal to trigger
         # population. To fix this another check was added to baseview
@@ -260,6 +266,9 @@ class Window(Gtk.ApplicationWindow):
 
         self._stack.set_visible_child(self.views[View.ALBUM])
 
+        self.views[View.SEARCH].bind_property(
+            'search-state', self._searchbar, 'search-state')
+
     @log
     def _select_all(self, action=None, param=None):
         if not self.props.selection_mode:
@@ -284,7 +293,7 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _on_key_press(self, widget, event):
-        modifiers = event.get_state()
+        modifiers = event.get_state() & Gtk.accelerator_get_default_mod_mask()
         (_, keyval) = event.get_keyval()
 
         control_mask = Gdk.ModifierType.CONTROL_MASK
@@ -386,15 +395,8 @@ class Window(Gtk.ApplicationWindow):
             self._searchbar.reveal(True)
 
     @log
-    def do_button_release_event(self, event):
-        """Override default button release event
-
-        :param Gdk.EventButton event: Button event
-        """
-        __, code = event.get_button()
-        # Mouse button 8 is the navigation button
-        if code == 8:
-            self._switch_back_from_childview()
+    def _on_back_button_pressed(self, gesture, n_press, x, y):
+        self.headerbar._on_back_button_clicked()
 
     @log
     def _notify_mode_disconnect(self, data=None):
@@ -407,12 +409,6 @@ class Window(Gtk.ApplicationWindow):
         self.prev_view = self.curr_view
         self.curr_view = stack.get_visible_child()
 
-        # Switch to all albums view when we're clicking Albums
-        if (self.curr_view == self.views[View.ALBUM]
-                and not (self.prev_view == self.views[View.SEARCH]
-                    or self.prev_view == self.views[View.EMPTY])):
-            self.curr_view.set_visible_child(self.curr_view._grid)
-
         if (self.curr_view != self.views[View.SEARCH]
                 and self.curr_view != self.views[View.EMPTY]):
             self._searchbar.reveal(False)
@@ -460,8 +456,8 @@ class Window(Gtk.ApplicationWindow):
                     and child != self.curr_view._artist_albums_widget):
                 self._stack.set_visible_child(self.views[View.ALBUM])
 
-            if self.props.selection_mode:
-                self.props.selection_mode = False
+        if self.props.selection_mode:
+            self.props.selection_mode = False
 
     @log
     def _switch_back_from_childview(self, klass=None):
@@ -475,9 +471,6 @@ class Window(Gtk.ApplicationWindow):
         if self.curr_view in views_with_child:
             self.curr_view._back_button_clicked(self.curr_view)
 
-        if self.curr_view != self.views[View.SEARCH]:
-            self._searchbar.reveal(False)
-
     @log
     def _on_selection_mode_changed(self, widget, data=None):
         if self.props.selection_mode:
